@@ -11,7 +11,8 @@ import sys
 from typing import TYPE_CHECKING, Any, Collection
 
 if TYPE_CHECKING:
-    from .parameters import Parameter, ParamOrGroup
+    from .parameters import Parameter
+    from .typing import ParamOrGroup
 
 __all__ = [
     'CommandParserException',
@@ -27,6 +28,7 @@ __all__ = [
     'ParamConflict',
     'ParamsMissing',
     'NoActiveContext',
+    'NoEnvVar',
 ]
 
 
@@ -177,4 +179,8 @@ class Backtrack(CommandParserException):
     """Raised when backtracking took place"""
 
 
+class NoEnvVar(CommandParserException):
+    """Indicates that no environment variable was available for a given Parameter."""
+
+
 # endregion
