@@ -4,21 +4,26 @@ Custom input handlers for Parameters to restrict allowed values to a set of choi
 :author: Doug Skrypa
 """
 
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
 from enum import Enum
-from typing import Any, Type, TypeVar, Collection, Iterator, Optional, Set, Mapping
+from typing import TYPE_CHECKING, Any, Type, TypeVar, Collection, Iterator, Optional, Set, Mapping
 
-from ..utils import Bool
-from .base import InputType, TypeFunc
+from ..typing import TypeFunc, T
+from .base import InputType
 from .exceptions import InvalidChoiceError
 
+if TYPE_CHECKING:
+    from ..utils import Bool
+
 __all__ = ['Choices', 'ChoiceMap', 'EnumChoices']
 
 EnumT = TypeVar('EnumT', bound=Enum)
 
 
-class _ChoicesBase(InputType, ABC):
-    choices: Collection[Any]
+class _ChoicesBase(InputType[T], ABC):
+    choices: Collection[T]
     type: Optional[TypeFunc] = None
     case_sensitive: bool = True
 
@@ -40,7 +45,7 @@ class _ChoicesBase(InputType, ABC):
     def _choices_repr(self, delim: str = ',') -> str:
         raise NotImplementedError
 
-    def _normalize(self, value: str) -> Any:
+    def _normalize(self, value: str) -> T:
         if self.type is not None:
             try:
                 return self.type(value)  # pylint: disable=E1102
@@ -48,13 +53,13 @@ class _ChoicesBase(InputType, ABC):
                 raise InvalidChoiceError(value, self.choices) from e
         return value
 
-    def _iter_normalized(self, value: Any, choices: Collection = None) -> Iterator[Any]:
+    def _iter_normalized(self, value: Any, choices: Collection = None) -> Iterator[T]:
         yield value
         if not self.case_sensitive and (choices is None or isinstance(choices, (Set, Mapping))):
             yield value.lower()
             yield value.upper()
 
-    def _case_insensitive_map_choice(self, value: Any) -> Any:
+    def _case_insensitive_map_choice(self, value: Any) -> T:
         if not self.case_sensitive:
             norm_value = value.casefold()
             for choice, val in self.choices.items():  # noqa
@@ -67,7 +72,7 @@ class _ChoicesBase(InputType, ABC):
         return '{{{}}}'.format(choice_delim.join(map(str, self.choices)))
 
 
-class Choices(_ChoicesBase):
+class Choices(_ChoicesBase[T]):
     """
     Validates that values are members of the collection of allowed values.
 
@@ -78,7 +83,7 @@ class Choices(_ChoicesBase):
       all strings, then this cannot be set to False.
     """
 
-    def __init__(self, choices: Collection[Any], type: TypeFunc = None, case_sensitive: Bool = True):  # noqa
+    def __init__(self, choices: Collection[T], type: TypeFunc = None, case_sensitive: Bool = True):  # noqa
         if not case_sensitive and not all(isinstance(c, str) for c in choices):
             raise TypeError(f'Cannot combine case_sensitive=False with non-str choices={choices}')
         elif isinstance(type, EnumChoices) and not any(isinstance(c, type.enum) for c in choices):
@@ -91,7 +96,7 @@ class Choices(_ChoicesBase):
     def _choices_repr(self, delim: str = ',') -> str:
         return delim.join(map(repr, sorted(self.choices)))
 
-    def __call__(self, value: str) -> Any:
+    def __call__(self, value: str) -> T:
         choices = self.choices
         value = self._normalize(value)
         for val in self._iter_normalized(value, choices):
@@ -107,7 +112,7 @@ class Choices(_ChoicesBase):
         raise InvalidChoiceError(value, choices)
 
 
-class ChoiceMap(Choices):
+class ChoiceMap(Choices[T]):
     """
     Similar to :class:`Choices`, but requires a mapping for allowed values.
 
@@ -119,12 +124,12 @@ class ChoiceMap(Choices):
       all strings, then this cannot be set to False.
     """
 
-    choices: Mapping[Any, Any]
+    choices: Mapping[Any, T]
 
-    def __init__(self, choices: Mapping[Any, Any], *args, **kwargs):
+    def __init__(self, choices: Mapping[Any, T], *args, **kwargs):
         super().__init__(choices, *args, **kwargs)
 
-    def __call__(self, value: str) -> Any:
+    def __call__(self, value: str) -> T:
         value = self._normalize(value)
         for val in self._iter_normalized(value):
             try:
@@ -135,7 +140,7 @@ class ChoiceMap(Choices):
         return self._case_insensitive_map_choice(value)
 
 
-class EnumChoices(_ChoicesBase):
+class EnumChoices(_ChoicesBase[EnumT]):
     """
     Similar to :class:`ChoiceMap`, but uses an Enum to validate / normalize input instead of the keys in a dict.
 
