@@ -12,7 +12,7 @@ from abc import ABC, abstractmethod
 from contextvars import ContextVar
 from functools import partial, update_wrapper
 from itertools import chain
-from typing import TYPE_CHECKING, Any, Type, Optional, Callable, Collection, Union, List, FrozenSet
+from typing import TYPE_CHECKING, Any, Type, Generic, Optional, Callable, Collection, Union, List, FrozenSet
 
 try:
     from functools import cached_property  # pylint: disable=C0412
@@ -20,13 +20,15 @@ except ImportError:
     from ..compat import cached_property
 
 from ..config import CommandConfig, OptionNameMode
-from ..context import Context, ctx, get_current_context
+from ..context import Context, ctx, get_current_context, ParseState
 from ..exceptions import ParameterDefinitionError, BadArgument, MissingArgument, InvalidChoice
 from ..exceptions import ParamUsageError, NoActiveContext, UnsupportedAction
-from ..inputs import InputType, normalize_input_type, Choices, ChoiceMap as ChoiceMapInput
+from ..inputs import InputType, normalize_input_type
+from ..inputs.choices import _ChoicesBase, Choices, ChoiceMap as ChoiceMapInput
 from ..inputs.exceptions import InputValidationError, InvalidChoiceError
 from ..nargs import Nargs
-from ..utils import _NotSet, Bool, get_descriptor_value_type
+from ..typing import Bool, T_co
+from ..utils import _NotSet, get_descriptor_value_type
 from .option_strings import OptionStrings
 
 if TYPE_CHECKING:
@@ -39,6 +41,7 @@ if TYPE_CHECKING:
 __all__ = ['Parameter', 'BasePositional', 'BaseOption']
 
 _group_stack = ContextVar('cli_command_parser.parameters.base.group_stack', default=[])
+# TODO: Remove the dependency for storing self.command?
 
 
 class parameter_action:  # pylint: disable=C0103
@@ -181,7 +184,7 @@ class ParamBase(ABC):
     # endregion
 
 
-class Parameter(ParamBase, ABC):
+class Parameter(ParamBase, Generic[T_co], ABC):
     """
     Base class for all other parameters.  It is not meant to be used directly.
 
@@ -196,8 +199,6 @@ class Parameter(ParamBase, ABC):
     :param required: Whether this parameter is required or not.  If it is required, then an exception will be
       raised if the user did not provide a value for this parameter.  Defaults to ``False``.
     :param metavar: The name to use as a placeholder for values in usage / help messages.
-    :param choices: A container that holds the specific values that users must pick from.  By default, any value is
-      allowed.
     :param help: A brief description of this parameter that will appear in ``--help`` text.
     :param hide: If ``True``, this parameter will not be included in usage / help messages.  Defaults to ``False``.
     :param show_default: Override the :attr:`.CommandConfig.show_defaults` setting for this parameter to always or
@@ -206,15 +207,16 @@ class Parameter(ParamBase, ABC):
 
     # region Attributes & Initialization
 
-    _actions: FrozenSet[str] = frozenset()
-    _positional: bool = False
-    _repr_attrs: Optional[Collection[str]] = None
-    accepts_none: bool = False
-    accepts_values: bool = True
-    choices: Optional[Collection[Any]] = None
+    # Class attributes
+    _actions: FrozenSet[str] = frozenset()          #: The actions supported by this Parameter
+    _positional: bool = False                       #: Whether this Parameter is positional or not
+    _repr_attrs: Optional[Collection[str]] = None   #: Attributes to include in ``repr()`` output
+    accepts_none: bool = False                      #: Whether this Parameter can be provided without a value
+    accepts_values: bool = True                     #: Whether this Parameter can be provided with at least 1 value
+    # Instance attributes with class defaults
     metavar: str = None
-    nargs: Nargs = Nargs(1)
-    type: Optional[Callable[[str], Any]] = None
+    nargs: Nargs = Nargs(1)                         # Set in subclasses
+    type: Optional[Callable[[str], T_co]] = None    # Only set here if not set by __init__ in Option/Positional
     show_default: bool = None
 
     def __init_subclass__(
@@ -250,7 +252,6 @@ class Parameter(ParamBase, ABC):
         default: Any = _NotSet,
         required: Bool = False,
         metavar: str = None,
-        choices: Collection[Any] = None,
         help: str = None,  # noqa
         hide: Bool = False,
         show_default: Bool = None,
@@ -259,8 +260,6 @@ class Parameter(ParamBase, ABC):
             raise ParameterDefinitionError(
                 f'Invalid action={action!r} for {self.__class__.__name__} - valid actions: {sorted(self._actions)}'
             )
-        if not choices and choices is not None:
-            raise ParameterDefinitionError(f'Invalid choices={choices!r} - when specified, choices cannot be empty')
         if required and default is not _NotSet:
             raise ParameterDefinitionError(
                 f'Invalid combination of required=True with default={default!r} for {self.__class__.__name__} -'
@@ -268,28 +267,33 @@ class Parameter(ParamBase, ABC):
             )
         super().__init__(name=name, required=required, help=help, hide=hide)
         self.action = action
-        self.choices = choices
         self.default = None if default is _NotSet and not required and self.nargs.max == 1 else default
         self.metavar = metavar
         if show_default is not None:
             self.show_default = show_default
 
-    @staticmethod
-    def _init_value_factory():
+    def _init_value_factory(self, state: ParseState):
         return _NotSet
 
     def __set_name__(self, command: CommandType, name: str):
         super().__set_name__(command, name)
         type_attr = self.type
         choices = isinstance(type_attr, (ChoiceMapInput, Choices)) and type_attr.type is None
-        if choices or type_attr is None:
-            annotated_type = get_descriptor_value_type(command, name)
-            if annotated_type is not None:
-                if choices:
-                    type_attr.type = annotated_type
-                else:  # self.type must be None
-                    # Choices present earlier would have already been converted
-                    self.type = normalize_input_type(annotated_type, None)
+        if not (choices or type_attr is None):
+            return
+        annotated_type = get_descriptor_value_type(command, name)
+        if annotated_type is None:
+            return
+        elif choices:
+            type_attr.type = annotated_type
+        else:  # self.type must be None
+            # Choices present earlier would have already been converted
+            self.type = normalize_input_type(annotated_type, None)
+
+    @property
+    def has_choices(self) -> bool:
+        type_attr = self.type
+        return isinstance(type_attr, _ChoicesBase) and type_attr.choices
 
     # endregion
 
@@ -339,9 +343,7 @@ class Parameter(ParamBase, ABC):
 
         ctx.record_action(self)
         action_method = getattr(self, action)
-        normalized = self.prepare_value(value, short_combo) if value is not None else value
-        self.validate(normalized)
-        return action_method(normalized)
+        return action_method(self.prepare_and_validate(value, short_combo))
 
     def would_accept(self, value: str, short_combo: bool = False) -> bool:
         action = self.action
@@ -355,9 +357,15 @@ class Parameter(ParamBase, ABC):
             return False
         return self.is_valid_arg(normalized)
 
+    def prepare_and_validate(self, value: str, short_combo: bool = False) -> T_co:
+        if value is not None:
+            value = self.prepare_value(value, short_combo)
+        self.validate(value)
+        return value
+
     def prepare_value(  # pylint: disable=W0613
         self, value: str, short_combo: bool = False, pre_action: bool = False
-    ) -> Any:
+    ) -> T_co:
         type_func = self.type
         if type_func is None or (pre_action and isinstance(type_func, InputType) and type_func.is_valid_type(value)):
             return value
@@ -372,7 +380,7 @@ class Parameter(ParamBase, ABC):
         except Exception as e:
             raise BadArgument(self, f'unable to cast value={value!r} to type={type_func!r}') from e
 
-    def validate(self, value: Any):
+    def validate(self, value: Optional[T_co]):
         if isinstance(value, str) and value.startswith('-'):
             if len(value) > 1 and not _is_numeric(value):
                 raise BadArgument(self, f'invalid value={value!r}')
@@ -390,13 +398,25 @@ class Parameter(ParamBase, ABC):
         else:
             return True
 
-    def result_value(self) -> Any:
+    def _fix_default(self, value) -> Optional[T_co]:
+        type_func = self.type
+        if type_func is not None and isinstance(type_func, InputType):
+            return type_func.fix_default(value)
+        return value
+
+    def _fix_default_collection(self, values) -> Optional[T_co]:
+        type_func = self.type
+        if type_func is None or not isinstance(type_func, InputType) or not isinstance(values, (list, tuple, set)):
+            return values
+        return values.__class__(map(type_func.fix_default, values))
+
+    def result_value(self) -> Optional[T_co]:
         value = ctx.get_parsed_value(self)
         if value is _NotSet:
             if self.required:
                 raise MissingArgument(self)
             else:
-                return self.default
+                return self._fix_default(self.default)
 
         if self.action == 'store':
             return value
@@ -406,9 +426,9 @@ class Parameter(ParamBase, ABC):
             default = self.default
             if default is not _NotSet:
                 if isinstance(default, Collection) and not isinstance(default, str):
-                    value = default
+                    value = self._fix_default_collection(default)
                 else:
-                    value.append(default)
+                    value.append(self._fix_default(default))
 
         nargs = self.nargs
         val_count = len(value)
@@ -447,12 +467,17 @@ class BasicActionMixin:
     action: str
     nargs: Nargs
 
+    def _init_value_factory(self, state: ParseState):
+        if self.action == 'append':
+            return []
+        return super()._init_value_factory(state)  # noqa
+
     @parameter_action
-    def store(self: Parameter, value: Any):
+    def store(self: Parameter, value: T_co):
         ctx.set_parsed_value(self, value)
 
     @parameter_action
-    def append(self: Parameter, value: Any):
+    def append(self: Parameter, value: T_co):
         ctx.get_parsed_value(self).append(value)
 
     def _pre_pop_values(self: Parameter):
@@ -477,7 +502,7 @@ class BasicActionMixin:
         if not values:
             return values
 
-        ctx.set_parsed_value(self, self._init_value_factory())
+        ctx.set_parsed_value(self, self._init_value_factory(ctx.state))
         ctx._provided[self] = 0
         return values
 
@@ -491,7 +516,7 @@ class BasicActionMixin:
         return values[-count:]
 
 
-class BasePositional(Parameter, ABC):
+class BasePositional(Parameter[T_co], ABC):
     """
     Base class for :class:`.Positional`, :class:`.SubCommand`, :class:`.Action`, and any other parameters that are
     provided positionally, without prefixes.  It is not meant to be used directly.
@@ -531,7 +556,7 @@ class BasePositional(Parameter, ABC):
         super().__init__(action, default=default, required=required, **kwargs)
 
 
-class BaseOption(Parameter, ABC):
+class BaseOption(Parameter[T_co], ABC):
     """
     Base class for :class:`.Option`, :class:`.Flag`, :class:`.Counter`, and any other keyword-like parameters that have
     ``--long`` and ``-short`` prefixes before values.
