@@ -11,15 +11,16 @@ from string import whitespace, printable
 from typing import TYPE_CHECKING, Any, Type, Optional, Callable, Union, Collection, Mapping, Dict
 from types import MethodType
 
-from ..context import ctx
+from ..context import ctx, ParseState
 from ..exceptions import ParameterDefinitionError, BadArgument, MissingArgument, InvalidChoice, CommandDefinitionError
 from ..formatting.utils import format_help_entry
 from ..nargs import Nargs
-from ..utils import _NotSet, Bool, camel_to_snake_case
+from ..utils import _NotSet, camel_to_snake_case
 from .base import BasePositional, parameter_action
 
 if TYPE_CHECKING:
     from ..core import CommandType
+    from ..typing import Bool
 
 __all__ = ['SubCommand', 'Action']
 
@@ -91,19 +92,20 @@ class ChoiceMap(BasePositional):
             cls._choice_validation_exc = choice_validation_exc
 
     def __init__(self, *, action: str = 'append', title: str = None, description: str = None, **kwargs):
-        choices = kwargs.setdefault('choices', None)
-        if choices is not None:
-            raise ParameterDefinitionError(
-                f'Invalid choices={choices!r} - {self.__class__.__name__} choices must be added via register'
-            )
         super().__init__(action=action, **kwargs)
-        self._init_value_factory = list
         self.title = title
         self.description = description
         self.choices = {}
 
+    def _init_value_factory(self, state: ParseState):
+        return []
+
     # region Choice Registration
 
+    @property
+    def has_choices(self) -> bool:
+        return bool(self.choices)
+
     def _update_nargs(self):
         try:
             lengths = set(map(len, map(str.split, self.choices)))
